<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013 Michael Knoll <knoll@punkt.de>, punkt.de GmbH
 *
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/


/**
 * Class implements an instances container for configuration builder instances.
 *
 * TODO refactor this into an abstract class that holds basic functionality
 *
 * This class holds the identifier-based singleton instances of the configuration builders.
 *
 * @author Michael Knoll <knoll@punkt.de>
 * @package Domain
 * @subpackage  Configuration
 * @see
 */
class Tx_PtExtlist_Domain_Configuration_ConfigurationBuilderInstancesContainer implements \TYPO3\CMS\Core\SingletonInterface {

	/**
	 * Holds an array of data backend instances as list-identifier based singletons
	 *
	 * @var array<Tx_PtExtlist_Domain_Configuration_ConfigurationBuilder>
	 */
	private $instances = array();



	/**
	 * Adds a given configuration builder to the instances of this container.
	 *
	 * Use set() if you want to overwrite existing instances.
	 *
	 * @param Tx_PtExtlist_Domain_Configuration_ConfigurationBuilder $configurationBuilder
	 * @throws Exception if a configuration builder for this list identifier has already been added to this container.
	 */
	public function add(Tx_PtExtlist_Domain_Configuration_ConfigurationBuilder $configurationBuilder) {
		if (empty($this->instances[$configurationBuilder->getListIdentifier()])) {
			$this->instances[$configurationBuilder->getListIdentifier()] = $configurationBuilder;
		} else {
			throw new Exception('A configuration builder for this list identifier has already been added. Please check first, whether this container already has an instance for the given list identifier! 1363856875');
		}
	}



	/**
	 * Sets a given configuration builder as instance. Overwrites already set instance.
	 *
	 * @param Tx_PtExtlist_Domain_Configuration_ConfigurationBuilder $configurationBuilder
	 */
	public function set(Tx_PtExtlist_Domain_Configuration_ConfigurationBuilder $configurationBuilder) {
		$this->instances[$configurationBuilder->getListIdentifier()] = $configurationBuilder;
	}



	/**
	 * Returns true, if a configuration builder for given list identifier exists.
	 *
	 * @param $listIdentifier
	 * @return bool
	 */
	public function contains($listIdentifier) {
		return array_key_exists($listIdentifier, $this->instances);
	}



	/**
	 * Removes a configuration builder for the given list identifier.
	 *
	 * @param string $listIdentifier
	 */
	public function remove($listIdentifier) {
		if ($this->contains($listIdentifier)) {
			unset($this->instances[$listIdentifier]);
		}
	}



	/**
	 * Returns instance of configuration builder for given list identifier
	 *
	 * @param $listIdentifier
	 * @return null|Tx_PtExtlist_Domain_Configuration_ConfigurationBuilder
	 */
	public function get($listIdentifier) {
		if (array_key_exists($listIdentifier, $this->instances)) {
			return $this->instances[$listIdentifier];
		} else {
			return NULL;
		}
	}

}